//+------------------------------------------------------------------+
//|                                                      DowGap.mq4 |
//|                                                       Tony Smith |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Tony Smith"
#property link      ""
#property version   "1.00"
#property strict

/*#include <Common/Defines.mqh>
#include <Common/Logging.mqh>
#include <Common/Date.mqh>
#include <Common/Aborter.mqh>
#include <Common/ParsePeriod.mqh>
#include <Common/Emailer.mqh>

#include <Trading/TradeManager.mqh>
#include <Trading/Order.mqh>
#include <Trading/TradeSizeCalculator.mqh>
*/

#include "SaadIncl.mqh"

input string SymbToTrade = "US30()"; // Symbol to trade
input string CandlestickLength = "1M"; // Candlestick length (1M, 5M, 1H, 1D etc)
input string StartTime = "14:30"; // Start Trading Time (HH:MM)
input string EndTime = "21:00"; // End Trading Time (HH:MM)
input double MinGap = 5.0; // Minimum Gap (in points) To Trade
input double OpenCloseAdjustment = 2.0; // Extra gap to move open/close
input uint InitCandlestickLength = 5; // Initial candlestick length (in chart bars)
input double InitStopLoss = -52.0; // Initial stoploss
input string OrderExpiry = "20:50"; // Order expires at (HH:MM)
input double TakeHalfPoints = 25.0; // Points to take some profits 
input double TakeHalfAmount = 50.0; // Percentage of profits to take
input double TakeHalfStop = -20.0; // New stoploss after taking profits
input string StopTighteningTime = "17:00"; // Stop Tightening (ST) time (HH:MM)
input double STProfitLock = 10.0; // ST Profit required to lock profits
input double STAmount = 50.0; // ST Percentage of profit to lock in
input double STProfitBE = 0.0; // ST Min Profit to set stops to break even
input double MinTradeSize = 2.0; // Minimum Trade Size
input double StartUsingRisk = 2000.0; // Balance to start sizing up trades
input double Risk = 5.0; // Percentage of account to risk
input double MinAccountBalance = 1000.0; // Min account balance warn level
input int Magic = 1001; // Magic Number
input bool ForcePlaceOrder = false; // Force EA to place order on startup
input string HolidayFile = "GER30Holidays.txt"; // Holiday file name

uint LoggingLevel = INFO; // Logging Level

PTradeManager pTradeManager;

class DowGap
{
private:
	CTime _startTime,_endTime,_orderTime,_orderExpiryTime,_stopTighteningTime;
	
	PDate _pCurrentDate;
	double _yesterdaysCloseLevel;
	bool _tradesClosed;
	bool _ordersPlaced;
	bool _stopTightened;
	bool _warnAccountLevel;
	bool _init;
	bool _noTradeToday;
	bool _takeHalfDone;
	PDate _holidays[];
	int _numHolidays;
	
	Logger _log;
	
public:
	DowGap() :
	_startTime(StartTime),
	_endTime(EndTime),
	_log("DowGap"),
	_orderExpiryTime(OrderExpiry),
	_stopTighteningTime(StopTighteningTime),
	_warnAccountLevel(false),
	_init(false),
	_takeHalfDone(false),
	_noTradeToday(true),
	_numHolidays(0)
	{
		_orderTime = _startTime;
		_orderTime.AddMins(InitCandlestickLength * Period());
		
		CTime timeNow(TimeCurrent());
		if ((timeNow >= _startTime) && !ForcePlaceOrder) _ordersPlaced = true;
		if (timeNow >= _stopTighteningTime) _stopTightened = true;
		
		ReadHolidays();
	}	
	
	void DoTick()
	{
		Date todaysDate(TimeCurrent());
		
		// check the day of the week.
		if ((todaysDate.GetDayOfWeek() == dayofweek_saturday) || (todaysDate.GetDayOfWeek() == dayofweek_sunday))
		{
			return;
		}
		
		if (!_init)
		{
			NewDay();
			_init = true;
		}
		
		// first check if we are on to a new day...
		if (todaysDate.NotEquals(_pCurrentDate.Val)) NewDay();
		
		if (_noTradeToday) return;
		
		// are we past end of trading time?
		CTime timeNow(TimeCurrent());
		if (!_tradesClosed && (timeNow >= _endTime))
		{
			_log.Write(INFO,"We have reached the end of trading - closing all positions.");
			pTradeManager.Val.RemoveOrdersAndCloseTrades(Magic);
			
			_tradesClosed = true;
		}
		
		// Is it time to place the first order?
		if (!_ordersPlaced && (timeNow >= _orderTime))
		{
			// Construct the candlestick..
			double low = 999999999.0;
			double high = 0.0;
			
			for (int i=1; i<=(int)InitCandlestickLength; i++)
			{
				if (Low[i] < low) low = Low[i];
				if (High[i] > high) high = High[i];
			}
			
			_log.Write(INFO,"Time to place order,  candlestick low/high is ",LOGLEVEL(low),"/",LOGLEVEL(high));
			
			// create buy/sell orders
			POrder pOrder(new Order());
			pOrder.Val.Colour = clrWhite;
			
			DateTime expiry(TimeCurrent());
			expiry.TimePart.Set(_orderExpiryTime);
			pOrder.Val.Expiry = expiry.Todatetime();
			
			pOrder.Val.Size = GetTradeSize();
			pOrder.Val.OrderType_ = ordertype_stop;
			pOrder.Val.StrategyID = Magic;
						
			// check there's enough of a gap..
			if (low > _yesterdaysCloseLevel)
			{
				if ((low - _yesterdaysCloseLevel) < MinGap)
				{
					_log.Write(INFO,"Gap from yesterdays close (",LOGLEVEL(_yesterdaysCloseLevel),") to todays candlestick low is less than ",LOGLEVEL(MinGap)," points, NO TRADES TODAY.");
					_ordersPlaced = true;
					return;
				}
				
				pOrder.Val.Direction = direction_sell;
				pOrder.Val.OpenLevel = low - OpenCloseAdjustment;
				pOrder.Val.LimitLevel = _yesterdaysCloseLevel + OpenCloseAdjustment;
				
				if (pOrder.Val.OpenLevel <= pOrder.Val.LimitLevel)
				{
					_log.Write(INFO,"Cannot enter SELL order,  open level (",LOGLEVEL(pOrder.Val.OpenLevel),") would be less than Take Profit level (",LOGLEVEL(pOrder.Val.LimitLevel),")");
					_ordersPlaced = true;
					return;
				}
			}
			else if (high < _yesterdaysCloseLevel)
			{
				if ((_yesterdaysCloseLevel - high) < MinGap)
				{
					_log.Write(INFO,"Gap from yesterdays close (",LOGLEVEL(_yesterdaysCloseLevel),") to todays candlestick high is less than ",LOGLEVEL(MinGap)," points, NO TRADES TODAY.");
					_ordersPlaced = true;
					return;
				}
				
				pOrder.Val.Direction = direction_buy;
				pOrder.Val.OpenLevel = high + OpenCloseAdjustment;
				pOrder.Val.LimitLevel = _yesterdaysCloseLevel - OpenCloseAdjustment;
				
				if (pOrder.Val.OpenLevel >= pOrder.Val.LimitLevel)
				{
					_log.Write(INFO,"Cannot enter BUY order,  open level (",LOGLEVEL(pOrder.Val.OpenLevel),") would be greater than Take Profit level (",LOGLEVEL(pOrder.Val.LimitLevel),")");
					_ordersPlaced = true;
					return;
				}
				
			}

			pOrder.Val.StopPoints(MathAbs(InitStopLoss));
			
			pTradeManager.Val.SendOrder(pOrder);
			
			_ordersPlaced = true;
		}
		
		// Go through orders and check points profit..
		POrder pOrders[];
		int num = pTradeManager.Val.AllTradesAndOrders(Magic,pOrders);
		
		if (!_takeHalfDone && (num > 0))
		{
			// Only expecting one order at posn 0
			POrder pOrder = pOrders[0];
			if (pOrder.Val.Active && (pOrder.Val.PipsProfit() >= TakeHalfPoints))
			{
				// We need to realise a proportion of the profits.
				pOrder.Val.Size = TradeSizeCalculator::NormaliseSize(pOrder.Val.Size * TakeHalfAmount / 100.0);
				
				_log.Write(INFO,"Trade has reached ",LOGLEVEL(TakeHalfPoints)," pnts profit,  closing ",LOGSIZE(pOrder.Val.Size)," of the trade.");
				pTradeManager.Val.CloseTrade( pOrder);
		
				_takeHalfDone = true;
			}
			
			if (_takeHalfDone)
			{
				// The a new trade will have been created after partial close - so we need to fetch it again.
				ArrayFree(pOrders);
			
				num = pTradeManager.Val.AllTradesAndOrders(Magic,pOrders);
				if (num > 0)
				{
					pOrder = pOrders[0];
					
					_log.Write(INFO,"Moving stop of remaining amount to ", LOGLEVEL(TakeHalfStop), " points");
					pOrder.Val.StopPoints(MathAbs(TakeHalfStop));
					pTradeManager.Val.AmendTrade(pOrder);
				}
			}
		}

		// Is it stop tightening time?
		if (!_stopTightened && (timeNow >= _stopTighteningTime))
		{
			_log.Write(INFO,"It's ",StopTighteningTime," - time to tighten Stops");
			_stopTightened = true;
			
			// Go through orders and check points profit..
			ArrayFree(pOrders);
			num = pTradeManager.Val.AllTradesAndOrders(Magic,pOrders);
		
			if (num > 0)
			{
				POrder pOrder = pOrders[0];
			
				_log.Write(INFO,"Looking at order ",pOrder.Val.Summary()," Pips Profit = ",LOGLEVEL(pOrder.Val.PipsProfit()));
				
				if (pOrder.Val.Active)
				{
					if (pOrder.Val.PipsProfit() >= STProfitLock)
					{
						double dPointsToLock = pOrder.Val.PipsProfit() * STAmount / 100.0;
						
						_log.Write(INFO,"Trade has more than ",LOGLEVEL(STProfitLock)," pnts profit - locking in ",LOGLEVEL(dPointsToLock)," points.");
						
						if (pOrder.Val.Direction EQ direction_buy) pOrder.Val.StopLevel = pOrder.Val.OpenLevel + dPointsToLock;
						else pOrder.Val.StopLevel = pOrder.Val.OpenLevel - dPointsToLock;
						
						pTradeManager.Val.AmendTrade(pOrder);
					}
					else if (pOrder.Val.PipsProfit() >= STProfitBE)
					{
						_log.Write(INFO,"Trade has ",LOGLEVEL(pOrder.Val.PipsProfit())," points profit - setting stop to break even");
						
						pOrder.Val.StopLevel = pOrder.Val.OpenLevel;
						
						pTradeManager.Val.AmendTrade(pOrder);
					}
				}
			
			}
		}
		
		// Do we need to warn about account level?
		if ((AccountBalance() < MinAccountBalance) && !_warnAccountLevel)
		{
			Emailer emailer(true);
			emailer.AddText(StringConcatenate("Your account balance is now ",LOGSIZE(AccountBalance())));
			emailer.Send(StringConcatenate("You account balance is below ,",LOGSIZE(MinAccountBalance)));
		}
		else if (_warnAccountLevel && (AccountBalance() > MinAccountBalance)) _warnAccountLevel = false;
	}
	
private:
	void NewDay()
	{
		_log.Write(INFO,"New day trading initialising.");
		_pCurrentDate.Assign(new Date(TimeCurrent()));
		DateTime yesterdaysClose(TimeCurrent());
		yesterdaysClose.AddDays(-1);
		
		while ((yesterdaysClose.DatePart.GetDayOfWeek() == dayofweek_saturday) || (yesterdaysClose.DatePart.GetDayOfWeek() == dayofweek_sunday) || IsHoliday(yesterdaysClose.DatePart)) yesterdaysClose.AddDays(-1);
		
		yesterdaysClose.TimePart.Set(_endTime);
		
		datetime dtYesterday = yesterdaysClose.Todatetime();
		
		int i=0;
		while (i < Bars && Time[i] >= dtYesterday) i++;
		
		if (i EQ Bars)
		{
			_log.Write(INFO,"Cannot fetch yesterdays close price - not enough data.  NO TRADING TODAY.");
			_noTradeToday = true;
			return;
		}
		
		if (IsHoliday(_pCurrentDate.Val))
		{
			_log.Write(INFO,"Today has been marked as a holiday - NO TRADING TODAY.");
			_noTradeToday = true;
			return;
		}
		
		_noTradeToday = false;
		
		_yesterdaysCloseLevel = Close[i];
		_log.Write(INFO,"Yesterdays close level is ", LOGLEVEL(_yesterdaysCloseLevel));	
		
		_tradesClosed = false;
		_ordersPlaced = false;
		_stopTightened = false;
		_takeHalfDone = false;
	}
	
	double GetTradeSize()
	{
		if (AccountBalance() < StartUsingRisk) return TradeSizeCalculator::NormaliseSize(MinTradeSize);
		
		return TradeSizeCalculator::NormaliseSize((AccountBalance() * Risk / 100.0) / MathAbs(InitStopLoss));
	}
	
	void ReadHolidays()
	{
		if (!FileIsExist(HolidayFile))
		{
			_log.Write(INFO,"Cannot find holiday file ",HolidayFile," assuming no holidays.");
			return;
		}
		
		int fh = FileOpen(HolidayFile,FILE_READ);
		if (fh EQ INVALID_HANDLE)
		{
			_log.Write(CRIT,"Cannot open Holiday file: ",HolidayFile);
			return;
		}
		
		while (!FileIsEnding(fh))
		{
			string date = FileReadString(fh);
			PDate pDate(new Date(date));
			_numHolidays++;
			CDynamicArray::Resize(_holidays,_numHolidays);
			_holidays[_numHolidays -1] = pDate;
		}
		
		FileClose(fh);
	}
	
	bool IsHoliday(Date &date)
	{
		for (int i=0; i<_numHolidays; i++)
		{
			if (_holidays[i].Val.Equals(date)) return true;
		}
		
		return false;
	}

};

DowGap DowGap;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   	Logger::DefaultLogLevel = LoggingLevel;
   	
   	Logger logg("init");
   	
   	logg.Write(INFO,"Symb:",Symbol(),":",SymbToTrade," Period:",IntegerToString(Period()),":",IntegerToString(ParsePeriod::Parse(CandlestickLength)));
   	
  		// Check it's running on right chart!
	if ((StringCompare(Symbol(),SymbToTrade) != 0) || (Period() != ParsePeriod::Parse(CandlestickLength)))
	{
		Aborter::Abort(StringConcatenate("Incorrect Symbol(",Symbol(),") or timeframe!"));
		return INIT_FAILED;
	}
	
	TradeManager::CustomerInit();
	pTradeManager.Assign(new TradeManager(false,false));


//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
	if (Aborter::Aborted) return;
	
	pTradeManager.Val.OnTick();
	
	DowGap.DoTick();
   
  }
//+------------------------------------------------------------------+
